Package: remotes
Title: R Package Installation from Remote Repositories, Including
        'GitHub'
Version: 2.5.0
Authors@R: c(
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = c("aut", "cre")),
    person("Jim", "Hester", role = "aut"),
    person("Hadley", "Wickham", role = "aut"),
    person("Winston", "Chang", role = "aut"),
    person("Martin", "Morgan", role = "aut"),
    person("Dan", "Tenenbaum", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd")),
    person("Ascent Digital Services", role = "cph")
  )
Description: Download and install R packages stored in 'GitHub', 'GitLab',
    'Bitbucket', 'Bioconductor', or plain 'subversion' or 'git'
    repositories.  This package provides the 'install_*' functions in
    'devtools'. Indeed most of the code was copied over from 'devtools'.
License: MIT + file LICENSE
URL: https://remotes.r-lib.org, https://github.com/r-lib/remotes#readme
BugReports: https://github.com/r-lib/remotes/issues
Depends: R (>= 3.0.0)
Imports: methods, stats, tools, utils
Suggests: brew, callr, codetools, covr, curl, git2r (>= 0.23.0), knitr,
        mockery, pingr, pkgbuild (>= 1.0.1), rmarkdown, rprojroot,
        testthat (>= 3.0.0), webfakes, withr
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.2.3
SystemRequirements: Subversion for install_svn, git for install_git
NeedsCompilation: no
Packaged: 2024-03-17 12:41:55 UTC; gaborcsardi
Author: Gábor Csárdi [aut, cre],
  Jim Hester [aut],
  Hadley Wickham [aut],
  Winston Chang [aut],
  Martin Morgan [aut],
  Dan Tenenbaum [aut],
  Posit Software, PBC [cph, fnd],
  Ascent Digital Services [cph]
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Repository: CRAN
Date/Publication: 2024-03-17 13:20:02 UTC
