Package: noremotes
Title: Tools to make developing R code easier
License: MIT
Description: Package description.
Author: Bugs Bunny
Maintainer: Bugs Bunny <bunny@acme.com>
Version: 1.0.0
Suggests: testthat
Imports:
  example,
  dtplyr
Remotes:
  url::https://www.example.com/,
  url::https://github.com/tidyverse/dtplyr

