# Generated by roxygen2: do not edit by hand

S3method(format,bioc_git2r_remote)
S3method(format,bioc_xgit_remote)
S3method(format,bitbucket_remote)
S3method(format,cran_remote)
S3method(format,git2r_remote)
S3method(format,github_remote)
S3method(format,gitlab_remote)
S3method(format,local_remote)
S3method(format,remotes)
S3method(format,url_remote)
S3method(format,xgit_remote)
S3method(github_resolve_ref,"NULL")
S3method(github_resolve_ref,default)
S3method(github_resolve_ref,github_pull)
S3method(github_resolve_ref,github_release)
S3method(print,package_deps)
S3method(remote_download,bioc_git2r_remote)
S3method(remote_download,bioc_xgit_remote)
S3method(remote_download,bitbucket_remote)
S3method(remote_download,git2r_remote)
S3method(remote_download,github_remote)
S3method(remote_download,gitlab_remote)
S3method(remote_download,local_remote)
S3method(remote_download,svn_remote)
S3method(remote_download,url_remote)
S3method(remote_download,xgit_remote)
S3method(remote_metadata,bioc_git2r_remote)
S3method(remote_metadata,bioc_xgit_remote)
S3method(remote_metadata,bitbucket_remote)
S3method(remote_metadata,git2r_remote)
S3method(remote_metadata,github_remote)
S3method(remote_metadata,gitlab_remote)
S3method(remote_metadata,local_remote)
S3method(remote_metadata,svn_remote)
S3method(remote_metadata,url_remote)
S3method(remote_metadata,xgit_remote)
S3method(remote_package_name,bioc_git2r_remote)
S3method(remote_package_name,bioc_xgit_remote)
S3method(remote_package_name,bitbucket_remote)
S3method(remote_package_name,cran_remote)
S3method(remote_package_name,default)
S3method(remote_package_name,git2r_remote)
S3method(remote_package_name,github_remote)
S3method(remote_package_name,gitlab_remote)
S3method(remote_package_name,local_remote)
S3method(remote_package_name,svn_remote)
S3method(remote_package_name,url_remote)
S3method(remote_package_name,xgit_remote)
S3method(remote_sha,bioc_git2r_remote)
S3method(remote_sha,bioc_xgit_remote)
S3method(remote_sha,bitbucket_remote)
S3method(remote_sha,cran_remote)
S3method(remote_sha,default)
S3method(remote_sha,git2r_remote)
S3method(remote_sha,github_remote)
S3method(remote_sha,gitlab_remote)
S3method(remote_sha,local_remote)
S3method(remote_sha,svn_remote)
S3method(remote_sha,url_remote)
S3method(remote_sha,xgit_remote)
S3method(update,package_deps)
export(add_metadata)
export(available_packages)
export(available_packages_reset)
export(available_packages_set)
export(bioc_install_repos)
export(bioc_version)
export(dev_package_deps)
export(download_version)
export(git_credentials)
export(github_pull)
export(github_release)
export(github_remote)
export(gitlab_pat)
export(install_bioc)
export(install_bitbucket)
export(install_cran)
export(install_deps)
export(install_dev)
export(install_git)
export(install_github)
export(install_gitlab)
export(install_local)
export(install_remote)
export(install_svn)
export(install_url)
export(install_version)
export(local_package_deps)
export(package_deps)
export(parse_github_repo_spec)
export(parse_github_url)
export(parse_repo_spec)
export(remote_download)
export(remote_metadata)
export(remote_package_name)
export(remote_sha)
export(standardise_dep)
export(system_requirements)
export(update_packages)
importFrom(stats,update)
importFrom(tools,file_ext)
importFrom(utils,available.packages)
importFrom(utils,compareVersion)
importFrom(utils,contrib.url)
importFrom(utils,install.packages)
importFrom(utils,read.delim)
